/*
 * Copyright (C) 2005 The Mir-coders group
 *
 * This file is part of Mir.
 *
 * Mir is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Mir is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Mir; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * In addition, as a special exception, The Mir-coders gives permission to link
 * the code of this program with  any library licensed under the Apache Software License.
 * You must obey the GNU General Public License in all respects for all of the code used
 * other than the above mentioned libraries.  If you modify this file, you may extend this
 * exception to your version of the file, but you are not obligated to do so.
 * If you do not wish to do so, delete this exception statement from your version.
 */
package mir.entity;

import mir.log.LoggerWrapper;
import mir.storage.Database;
import mir.storage.store.StorableObject;
import mir.storage.store.StoreContainerType;
import mir.storage.store.StoreIdentifier;
import mir.storage.store.StoreUtil;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 *
 * Container class for lists of Entities.
 * Now implements @see mir.database.store.StorableObject.
 *
 * @author rk
 * first version	27.6.1999
 *
 * @version 1.1 (cleaned up)
 */
public class EntityList implements StorableObject {
  protected LoggerWrapper logger;
  private List                entities = new ArrayList();
  private String              whereClause, orderClause;
  private Database            storage;
  private int                 offset, limit;
  private int                 nextOffset = -1;

  public EntityList(){
    logger = new LoggerWrapper("Entity.List");
  }

  public void setStorage(Database aStorage) {
    storage=aStorage;
  }

  public Database getStorage() {
    return storage;
  }

  public void setLimit(int aLimit) {
    limit = aLimit;
  }

  /**
   * Sets the WHERE clause that fetched the Entities of this EntityList from the database.
   *
   * @param wc The string that contains the WHERE clause
   */
  public void setWhere(String wc) {
    this.whereClause = wc;
  }

  /**
   * Sets the sorting criterium of this EntityList
   *
   * @param oc
   */
  public void setOrder(String oc) {
    this.orderClause = oc;
  }

  /**
   * Sets the offset
   *
   * @param i The offset
   */
  public void setOffset(int i) {
    offset = i;
  }

  /**
   * Returns the offset
   *
   * @return offset
   */
  public int getOffset() {
    return offset;
  }

  /**
   * Sets the offset of the next batch of Entities.
   *
   * @param i The next offset
   */
  public void setNextBatch(int i) {
    nextOffset = i;
  }

  /**
   * Returns the offset of the next batch of Entities.
   *
   * @return offset of the next batch
   */
  public int getNextBatch() {
    return nextOffset;
  }

  /**
   * Returns whether there is a next batch within the WHERE clause
   *
   * @return true if yes, false if no.
   */
  public boolean hasNextBatch() {
    return (nextOffset >= 0);
  }

  /**
   * Inserts an Entity into the EntityList.
   *
   * @param anEntity The entity to be inserted.
   */

  public void add (Entity anEntity) {
    if (anEntity!=null)
      entities.add(anEntity);
    else
      logger.warn("EntityList: add called with empty Entity");
  }


  /**
   * @return The number of Entities in the EntityList.
   */

  public int size() {
    return entities.size();
  }


  /**
   * Returns the element at position i in the EntityList as Entity
   * @param i the position of the element in question
   * @return The element at position i.
   */

  public Entity elementAt(int i) {
    return (Entity) entities.get(i);
  }



// Methods to implement StorableObject

  public Set getNotifyOnReleaseSet() { return null; }

  public StoreIdentifier getStoreIdentifier() {
    if ( storage!=null ) {
      return
        new StoreIdentifier(
                this, StoreContainerType.STOC_TYPE_ENTITYLIST,
                StoreUtil.getEntityListUniqueIdentifierFor( storage.getTableName(),
                whereClause, orderClause, offset, limit ));
    }
		logger.warn("EntityList could not return StoreIdentifier");
		
		return null;
  }

}
